import { createMap } from "./util";
class Target {
    constructor(path, matcher, delegate) {
        this.path = path;
        this.matcher = matcher;
        this.delegate = delegate;
    }
    to(target, callback) {
        const delegate = this.delegate;
        if (delegate && delegate.willAddRoute) {
            target = delegate.willAddRoute(this.matcher.target, target);
        }
        this.matcher.add(this.path, target);
        if (callback) {
            if (callback.length === 0) {
                throw new Error("You must have an argument in the function passed to `to`");
            }
            this.matcher.addChild(this.path, target, callback, this.delegate);
        }
    }
}
export class Matcher {
    constructor(target) {
        this.routes = createMap();
        this.children = createMap();
        this.target = target;
    }
    add(path, target) {
        this.routes[path] = target;
    }
    addChild(path, target, callback, delegate) {
        const matcher = new Matcher(target);
        this.children[path] = matcher;
        const match = generateMatch(path, matcher, delegate);
        if (delegate && delegate.contextEntered) {
            delegate.contextEntered(target, match);
        }
        callback(match);
    }
}
function generateMatch(startingPath, matcher, delegate) {
    function match(path, callback) {
        const fullPath = startingPath + path;
        if (callback) {
            callback(generateMatch(fullPath, matcher, delegate));
        }
        else {
            return new Target(fullPath, matcher, delegate);
        }
    }
    return match;
}
function addRoute(routeArray, path, handler) {
    let len = 0;
    for (let i = 0; i < routeArray.length; i++) {
        len += routeArray[i].path.length;
    }
    path = path.substr(len);
    const route = { path, handler };
    routeArray.push(route);
}
function eachRoute(baseRoute, matcher, callback, binding) {
    const routes = matcher.routes;
    const paths = Object.keys(routes);
    for (let i = 0; i < paths.length; i++) {
        const path = paths[i];
        const routeArray = baseRoute.slice();
        addRoute(routeArray, path, routes[path]);
        const nested = matcher.children[path];
        if (nested) {
            eachRoute(routeArray, nested, callback, binding);
        }
        else {
            callback.call(binding, routeArray);
        }
    }
}
export default function map(callback, addRouteCallback) {
    const matcher = new Matcher();
    callback(generateMatch("", matcher, this.delegate));
    eachRoute([], matcher, function (routes) {
        if (addRouteCallback) {
            addRouteCallback(this, routes);
        }
        else {
            this.add(routes);
        }
    }, this);
}
//# sourceMappingURL=dsl.js.map